package com.persagy.apm.translate.impl;

import cn.hutool.core.collection.CollUtil;
import com.persagy.apm.common.helper.SpringHelper;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.translate.extension.ITransWrapperBuilder;
import com.persagy.apm.translate.helper.TranslatorHelper;
import com.persagy.apm.translate.model.ITransWrapper;
import com.persagy.apm.translate.model.TransMeta;
import com.persagy.apm.translate.service.ITranslateBatchHandle;
import com.persagy.apm.translate.service.ITranslateHandle;
import com.persagy.apm.translate.service.ITranslateService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 翻译接口实现
 * @author Charlie Yu
 * @Date 2021-03-03
 */
@Service
@Transactional
public class TranslateServiceImpl implements ITranslateService {

    @SuppressWarnings("unchecked")
    @Override
    public Object translate(Object origins, List<TransMeta> metas) {
        if (origins == null || metas == null || metas.size() == 0) {
            return origins;
        }
        Object originObj = fetchTransObj(origins);
        if(originObj == null) {
            return origins;
        }
        //按翻译元数据生成翻译处理器
        List<ITranslateHandle> handles = new ArrayList<>();
        HashSet<String> properties = new HashSet<>();
        Class clazz = (originObj instanceof List) ? ((List<Object>) originObj).get(0).getClass() : originObj.getClass();
        ITransWrapperBuilder builder = TranslatorHelper.createTransWrapperBuilder(clazz);
        for (TransMeta meta : metas) {
            ITranslateHandle handle = createHandle(meta);
            handles.add(handle);
            meta.setBatch(meta.isBatch() && handle instanceof ITranslateBatchHandle);
            if (!builder.isNeedReplaceOrigin()) {
                continue;
            }
            //生成vo的扩展属性
            String target = meta.getTarget();
            if (StringUtils.isEmpty(target)) {
                continue;
            }
            if (target.indexOf(",") > -1) {
                String[] ps = target.split(",");
                for (String p : ps) {
                    properties.add(p);
                }
            } else {
                properties.add(target);
            }
        }
        builder.setExtendProps(properties);
        if (originObj instanceof List) {
            transMultiRows((List<Object>) originObj, handles, builder);
            return origins;
        } else {
            //单条数据翻译
            originObj = transSingleRow(originObj, handles, builder);
            if(origins instanceof CommonResult) {
                ((CommonResult) origins).setContent(originObj);
            }
            return origins;
        }
    }

    /**
     * 转换为实际待翻译对象
     * @param origins 原结果
     * @return
     */
    private Object fetchTransObj(Object origins){
        Object obj = null;
        //对分页和列表进行处理获取单个对象，并设置是否为列表标志
        if (origins instanceof Page) {
            try {
                Field field = PageImpl.class.getSuperclass().getDeclaredField("content");
                field.setAccessible(true);
                obj = (List<Object>) field.get(origins);
            } catch (NoSuchFieldException | IllegalAccessException e) {
                obj = null;
            }
        } else if (origins instanceof CommonResult) {
            obj = ((CommonResult) origins).getContent();
            if(obj instanceof PageList) {
                obj = ((PageList) obj).getRecords();
            }
        }
        if(obj == null) {
            return null;
        }
        return obj;
    }

    /**
     * 单个vo数据翻译
     *
     * @param data    单个数据
     * @param handles 翻译处理集合
     */
    protected Object transSingleRow(Object data, List<ITranslateHandle> handles, ITransWrapperBuilder builder) {
        ITransWrapper wrapper = builder.createWrapper(data);
        for (ITranslateHandle handle : handles) {
            handle.trans(wrapper);
        }
        return wrapper.getResult();
    }

    /**
     * 批量翻译
     *
     * @param datas   多行VO数据
     * @param handles 翻译处理集合
     * @return 翻译后的结果
     */
    protected List<Object> transMultiRows(List<Object> datas, List<ITranslateHandle> handles, ITransWrapperBuilder builder) {
        List<ITransWrapper> wrappers = new ArrayList<>();
        for (ITranslateHandle handle : handles) {
            TransMeta meta = handle.getMeta();
            if (meta.isBatch()) {
                batchTransMultiRows(datas, wrappers, (ITranslateBatchHandle) handle, builder);
            } else {
                transMultiRowLineByLine(datas, wrappers, handle, builder);
            }
        }
        return datas;
    }

    /**
     * 批量翻译vo数据
     *
     * @param datas    原始数据
     * @param wrappers 数据包装对象列表
     * @param handle   翻译处理
     * @param builder  　数据包装对象构建器
     */
    protected void batchTransMultiRows(List<Object> datas, List<ITransWrapper> wrappers, ITranslateBatchHandle handle, ITransWrapperBuilder builder) {
        List<String> parameters = new ArrayList<>();
        TransMeta meta = handle.getMeta();
        if (wrappers.size() == 0) {
            for (int i = 0; i < datas.size(); i++) {
                ITransWrapper wrapper = builder.createWrapper(datas.get(i));
                wrappers.add(wrapper);
                if (builder.isNeedReplaceOrigin()) {
                    datas.set(i, wrapper.getResult());
                }
                Object parameter = wrapper.getValue(meta.getParameter());
                if (parameter == null) {
                    continue;
                }
                if(parameter instanceof Collection) {
                    parameter = CollUtil.join(((Collection)parameter), ",");
                }
                parameters.add(parameter.toString());
            }
        } else {
            for (int i = 0; i < wrappers.size(); i++) {
                ITransWrapper wrapper = wrappers.get(i);
                Object parameter = wrapper.getValue(meta.getParameter());
                if (parameter == null) {
                    continue;
                }
                if(parameter instanceof Collection) {
                    parameter = CollUtil.join(((Collection)parameter), ",");
                }
                parameters.add(parameter.toString());
            }
        }
        parameters = parameters.stream().distinct().collect(Collectors.toList());
        Map<String, Object> batchRs = handle.batchTrans(parameters);
        if (batchRs == null) {
            return;
        }
        for (int i = 0; i < wrappers.size(); i++) {
            ITransWrapper wrapper = wrappers.get(i);
            Object parameter = wrapper.getValue(meta.getParameter());
            if (parameter == null) {
                continue;
            }
            Object rs = batchRs.get(parameter.toString());
            setRowTransResult(rs, handle.getMeta(), wrapper);
        }
    }

    /**
     * 逐行翻译vo数据
     *
     * @param datas    原始数据
     * @param wrappers 包装列表
     * @param handle   翻译处理
     * @param builder  　构建器
     */
    protected void transMultiRowLineByLine(List<Object> datas, List<ITransWrapper> wrappers, ITranslateHandle handle, ITransWrapperBuilder builder) {
        if (wrappers.size() == 0) {
            for (int i = 0; i < datas.size(); i++) {
                ITransWrapper wrapper = builder.createWrapper(datas.get(i));
                wrappers.add(wrapper);
                if (builder.isNeedReplaceOrigin()) {
                    datas.set(i, wrapper.getResult());
                }
            }
        }

        for (int j = 0; j < wrappers.size(); j++) {
            ITransWrapper wrapper = wrappers.get(j);
            handle.trans(wrapper);
        }
    }


    /**
     * 批量翻译时设置单行数据单个翻译结果
     *
     * @param rs      翻译结果
     * @param meta    翻译元数据
     * @param wrapper 包装对象
     */
    protected void setRowTransResult(Object rs, TransMeta meta, ITransWrapper wrapper) {
        if (rs == null) {
            return;
        }
        if (rs instanceof Map) {
            String[] rsFields = meta.getResultFields();
            String[] targetFields = meta.getTargetFields();
            Map<String, Object> resultMap = (Map<String, Object>) rs;
            int len = rsFields.length > targetFields.length ? targetFields.length : rsFields.length;
            for (int j = 0; j < len; j++) {
                Object rsr = resultMap.get(rsFields[j]);
                if (rsr != null) {
                    wrapper.setValue(targetFields[j], rsr.toString());
                }
            }
        } else {
            wrapper.setValue(meta.getTarget(), rs.toString());
        }
    }

    /**
     * 根据元数据生成处理器
     * @param meta 元数据
     * @return 处理器
     */
    private ITranslateHandle createHandle(TransMeta meta) {
        ITranslateHandle handle = SpringHelper.getBean(meta.getType(), ITranslateHandle.class);
        handle.setMeta(meta);
        return handle;
    }
}